from enum import Enum


class Manufacturer(Enum):
    """
    Enum holding all manufacturer names
    """

    BAT = "BAT"

    BRENNENSTUHL = "Brennenstuhl"

    ELRO = "Elro"

    HAMA = "hama"

    INTERTECHNO = "Intertechno"

    INTERTEK = "Intertek"

    LOGILINK = "LogiLink"

    LUX_GMBH = "Emil Lux GmbH"

    MUMBI = "Mumbi"

    M_E = "m-e GmbH modern-electronics"

    NONAME = "noname"

    POLLIN_ELECTRONIC = "Pollin Electronic"

    REV = "REV"

    SEEGEL_SYSTEME = "Seegel Systeme"

    SIMPLE_SOLUTIONS = "Simple Solutions"

    UNIVERSAL = "Universal"

    VIVANCO = "Vivanco"

    WESTFALIA = "Westfalia"
